#include <iostream>
#include <fstream>
#include <string>
#include <cmath>

#include "Rifle.h"

using namespace std;


Rifle::Rifle(double barrel_length, double bore, double groove_depth)
{
    m_barrel_length = barrel_length;
    m_bore = bore * 1e-3; // mm to m
    m_groove_depth = groove_depth * 1e-3; // mm to m 
    m_num_points = 0;
}


Rifle::~Rifle()
{
//    delete[] m_time;
//    delete[] m_length;
//    delete[] m_pressure;
//    delete[] m_velocity;
//    delete[] m_acceleration;
}


void Rifle::readPressure(const char *filename)
{
    string line;
	ifstream inFile;
	inFile.open(filename);
	
	double len, prs;
	
	//count lines
	int num_lines = 0;
	while ( getline(inFile, line) )
	{
		num_lines = num_lines + 1;
	}
	
	inFile.close();
	
	// allocate memory
	m_num_points = num_lines + 1;
	m_time = new double[m_num_points];
	m_length = new double[m_num_points];
	m_pressure = new double[m_num_points];
	m_velocity = new double[m_num_points];
	m_acceleration = new double[m_num_points];
	
	inFile.open(filename);
	
	// set values at L = 0
	m_time[0] = 0;
    m_length[0] = 0;
    m_pressure[0] = 0;
    m_velocity[0] = 0;
    m_acceleration[0] = 0;
    // read data
	num_lines = 1;
	while (inFile >> len >> prs)
	{
	    m_length[num_lines] = len;
	    m_pressure[num_lines] = prs * 1e9; // GPa to Pa
	    num_lines = num_lines + 1;
	}
	
	inFile.close();
}

// change in work = m * v^2 / 2
void Rifle::calculateVelocity(double bullet_mass)
{
    double area = M_PI * pow(m_bore/2.0, 2); // A = pi * R^2
    double work = 0;
    
    for (int i = 1; i < m_num_points; i++)
    {
        double h = m_length[i] - m_length[i - 1];
        // single integration step
        double integral = h * (m_pressure[i - 1] + m_pressure[i]) / 2;
        // force = area * (pressure integral)
        work = work + area * integral;
        m_velocity[i] = sqrt(2 * work / bullet_mass);
    }
}


// a = dv / dt
void Rifle::calculateAcceleration()
{
    for (int i = 1; i < m_num_points; i++)
    {
        double h = m_time[i] - m_time[i - 1];
        m_acceleration[i] = (m_velocity[i] - m_velocity[i - 1]) / h;
    }
}

//
// v(l) = dl / dt
// dl / v(l) = dt
// T = integral from 0 to L (1 / v dl)
void Rifle::calculateTime()
{
    double t = 0;
    for (int i = 2; i < m_num_points; i++)
    {
        double h = m_length[i] - m_length[i - 1];
        // single integration step
        double integral = h * (1 / m_velocity[i - 1] - 1 / m_velocity[i]) / 2;
        t = t + integral;
        m_time[i] = t;
    }
}

double Rifle::muzzleTime() const
{
    return m_time[m_num_points - 1];
}


double Rifle::muzzleVelocity() const
{
    return m_velocity[m_num_points - 1];
}


void Rifle::tabulateVelocity(ostream& os)
{
    for (int i = 0; i < m_num_points; ++i)
    {
        os << m_length[i] << "," << m_velocity[i] << endl;
    }
}


void Rifle::tabulateAcceleration(ostream& os)
{
    for (int i = 0; i < m_num_points; ++i)
    {
        os << m_length[i] << "," << m_acceleration[i] << endl;
    }
}
