#include <iostream>
#include <cmath>

#include "RifleRange.h"


RifleRange::RifleRange(double v0, double alpha)
{
    m_v0 = v0;
    m_alpha = alpha * M_PI / 180; //  deg to rad
}

double RifleRange::maxHeight() const
{
    return pow(m_v0 * sin(m_alpha), 2)  / (2 * g);
}

double RifleRange::maxTime() const
{
    return 2 * m_v0 * sin(m_alpha) / g;
}

double RifleRange::maxLength() const
{
    return m_v0 * m_v0 * sin(2 * m_alpha) / g;
}
    
    
void RifleRange::tabulateTrajectory(ostream& os, int num_points)
{
    double step = maxTime() / num_points;
    for (int i = 0; i <= num_points; ++i)
    {
        double t = i * step;
        double x = m_v0 * cos(m_alpha) * t;
        double y = m_v0 * sin(m_alpha) * t - g * t * t / 2;
        os << x << "," << y << endl;
    }
}

void RifleRange::tabulateVelocity(ostream& os, int num_points)
{
    double step = maxTime() / num_points;
    for (int i = 0; i <= num_points; ++i)
    {
        double t = i * step;
        double vx = m_v0 * cos(m_alpha);
        double vy = m_v0 * sin(m_alpha) - g * t;
        double v = sqrt(vx*vx + vy*vy);
        os << t << "," << v << endl;
    }
}
