#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include "Rifle.h"
#include "RifleRange.h"


using namespace std;

#define g 9.80665
#define BULLET_MASS (0.0956 / g)


string getDataFile(string rifleName)
{
    string fileName;
    cout << "Enter the file name for pressure data of " << rifleName << " rifle: ";
    cin >> fileName;
    
    //fileName = rifleName + ".txt";
    
    return fileName;
}


int main(int argc, char **argv)
{
    ofstream outFile;

    Rifle US1 = Rifle(0.6096, 7.62, 0.102);
    Rifle R1 = Rifle(0.4826, 8.5, 0.100);
    
    string US1_dataFile = getDataFile("US_1");
    string R1_dataFile = getDataFile("R_1");
    
    US1.readPressure(US1_dataFile.c_str());
    US1.calculateVelocity(BULLET_MASS);
    US1.calculateTime();
    US1.calculateAcceleration();
    
    
    R1.readPressure(R1_dataFile.c_str());
    R1.calculateVelocity(BULLET_MASS);
    R1.calculateTime();
    R1.calculateAcceleration();
    
    cout << "US_1         R_1" << endl; 
    cout << left << setw(7) << setprecision(2) 
         << US1.muzzleTime() * 1e6 << " ms    " 
         << R1.muzzleTime()  * 1e6 << " ms" << endl;
    cout << left << setw(7) << fixed << setprecision(2) 
         << US1.muzzleVelocity() << " m/s   " 
         << R1.muzzleVelocity()  << " m/s" << endl;
         
         
    outFile.open("bullet_velocity_US_1.txt");
    US1.tabulateVelocity(outFile);
    outFile.close();
    outFile.open("bullet_acceleration_US_1.txt");
    US1.tabulateAcceleration(outFile);
    outFile.close();
    
    outFile.open("bullet_velocity_R_1.txt");
    R1.tabulateVelocity(outFile);
    outFile.close();
    outFile.open("bullet_acceleration_R_1.txt");
    R1.tabulateAcceleration(outFile);
    outFile.close();
    
    
    
    cout << endl << "US_1" << endl;
    cout << "Angle    Time        Height      Length" << endl;
    for (int alpha = 25; alpha <= 55; alpha += 10)
    {
        RifleRange US1range(US1.muzzleVelocity(), alpha);
        cout << fixed << setprecision(2) << alpha << " deg " << setw(8) <<  US1range.maxTime() << " s " 
             << setw(8) << US1range.maxHeight() * 1e-3 << " km " 
             << setw(8) << US1range.maxLength() * 1e-3 << " km" << endl;
        stringstream ss;
        
	
        ss << "trajectory_US_1_" << alpha << ".txt";
        outFile.open(ss.str().c_str());
        US1range.tabulateTrajectory(outFile, 200);
        outFile.close();
        ss.str("");
        ss << "velocity_US_1_" << alpha << ".txt";
        outFile.open(ss.str().c_str());
        US1range.tabulateVelocity(outFile, 200);
        outFile.close();
    }
    
    cout << endl << "R_1" << endl;
    cout << "Angle    Time        Height     Length" << endl;
    for (int alpha = 25; alpha <= 55; alpha += 10)
    {
        RifleRange R1range(R1.muzzleVelocity(), alpha);
        cout << fixed << setprecision(2) << alpha << " deg " << setw(8) <<  R1range.maxTime() << " s " 
             << setw(8) << R1range.maxHeight() * 1e-3 << " km " 
             << setw(8) << R1range.maxLength() * 1e-3 << " km" << endl;
        
        stringstream ss;
	
        ss << "trajectory_R_1_" << alpha << ".txt";
        outFile.open(ss.str().c_str());
        R1range.tabulateTrajectory(outFile, 500);
        outFile.close();
        ss.str("");
        ss << "velocity_R_1_" << alpha << ".txt";
        outFile.open(ss.str().c_str());
        R1range.tabulateVelocity(outFile, 500);
        outFile.close();
    }

    return 0;
}
